% comsol_model. Author: Moges Retta, moges.retta@kuleuven.be
% Comsol linked with Matlab is required to run this file.
% Comsol version 5 and Matlab 2017 were used to write this code
% The code produces the solved model of gas diffusion and photosynthesis 
% kinetics. In addition, the response of photsynthesis to Irradiance (Iinc) 
% and intercellular CO2 (Ci) are produced. Inputs for comsol_model are 
% geometry information.These are matrices containig indices of cells, 
% vertices and  coordingates for each vertex of a cell. In addition, PID 
% and EID matrices specificy which of the cells belong to palisade and 
% epidermis. Cell ids that are not in EID and PID are then ids for spongy 
% cells.

function out = Comsol_model(coordinate,CellvertexIds,paliIds,epiIds,model_name,result_A_Ci,result_A_Iinc)


model = ModelUtil.create(model_name);
import com.comsol.model.*
import com.comsol.model.util.*
model.geom.create('geom1', 2);  % model dimension

% creating label for each cell type
model.geom('geom1').selection.create('csel1', 'CumulativeSelection');
model.geom('geom1').selection('csel1').label('Epidermis');

model.geom('geom1').selection.create('csel2', 'CumulativeSelection');
model.geom('geom1').selection('csel2').label('Palisade');

model.geom('geom1').selection.create('csel3', 'CumulativeSelection');
model.geom('geom1').selection('csel3').label('Spongy');

% Making polygons from x,y coordinates of the vertices of the cells and
% converting them to solid objects. These objects are then assined to each
% cell label shown above.

for i=1:length(CellvertexIds)
    
    x=coordinate(CellvertexIds{i},1);
    y=coordinate(CellvertexIds{i},2);
    x=x';
    y=y';
    XX = regexprep(num2str(x,17), '\s*', ',');
    YY= regexprep(num2str(y,17), '\s*', ',');

string=['csol',num2str(i)];
string1=['csol',num2str(length(CellvertexIds)+i)];

model.geom('geom1').feature.create(string, 'Polygon');
model.geom('geom1').feature(string).set('type', 'closed');
model.geom('geom1').feature(string).set('y', XX);
model.geom('geom1').feature(string).set('x', YY);
model.geom('geom1').run(string);

model.geom('geom1').create(string1, 'ConvertToSolid');
model.geom('geom1').feature(string1').selection('input').set({string});
model.geom('geom1').feature(string1').set('repairtol', '1.0E-6');
model.geom('geom1').run(string1');

model.geom('geom1').feature(string1).set('createselection', 'on');
model.geom('geom1').feature(string).set('createselection', 'on');

        if ismember(i,epiIds)
        model.geom('geom1').feature(string1).set('contributeto', 'csel1');
        elseif ismember(i,paliIds)
        model.geom('geom1').feature(string1).set('contributeto', 'csel2');
        else
        model.geom('geom1').feature(string1).set('contributeto', 'csel3');
        end
  
end

% correcly aligning the gometry in comsol geomery window
model.geom('geom1').create('rot1', 'Rotate');
model.geom('geom1').feature('rot1').selection('input').named('csel1');
model.geom('geom1').feature('rot1').set('rot', '90');
model.geom('geom1').run('rot1');

model.geom('geom1').create('rot2', 'Rotate');
model.geom('geom1').feature('rot2').selection('input').named('csel2');
model.geom('geom1').feature('rot2').set('rot', '90');
model.geom('geom1').run('rot2');

model.geom('geom1').create('rot3', 'Rotate');
model.geom('geom1').feature('rot3').selection('input').named('csel3');
model.geom('geom1').feature('rot3').set('rot', '90');
model.geom('geom1').run('rot3');

model.geom('geom1').create('mir1', 'Mirror');
model.geom('geom1').feature('mir1').selection('input').named('csel1');
model.geom('geom1').run('mir1');
model.geom('geom1').feature.duplicate('mir2', 'mir1');
model.geom('geom1').runPre('mir2');
model.geom('geom1').feature('mir2').selection('input').named('csel2');
model.geom('geom1').run('mir2');
model.geom('geom1').create('mir3', 'Mirror');
model.geom('geom1').feature('mir3').selection('input').named('csel3');
model.geom('geom1').run('mir3');

model.geom('geom1').runPre('fin');
model.geom('geom1').feature('fin').set('repairtol', '1.E-4');


% Adding a rectangular box encompasing the geometry
Xmin_= min(coordinate(:,1));
Ymin_= min(coordinate(:,2))-5;


Xmax_= max(coordinate(:,1));
Ymax_= max(coordinate(:,2))+10+abs(Ymin_); 
model.geom('geom1').create('r1', 'Rectangle');
model.geom('geom1').feature('r1').set('size', {num2str(Xmax_) num2str(Ymax_)});
model.geom('geom1').feature('r1').set('pos', {num2str(Xmin_) num2str(Ymin_)});
model.geom('geom1').run('r1');
model.geom('geom1').create('spl1', 'Split');
model.geom('geom1').feature('spl1').selection('input').set({'mir1' 'mir2' 'mir3' 'r1'});
model.geom('geom1').feature('spl1').set('keep', 'on');
model.geom('geom1').run('spl1');

model.geom('geom1').run('fin');
model.geom('geom1').lengthUnit([native2unicode(hex2dec({'00' 'b5'}), 'unicode') 'm']);

% Model parameters
model.param.set('P', '101325[Pa]', 'Total pressure');
model.param.set('T', '298[K]', 'Temperature');
model.param.set('R', '8.314[(m^3*Pa)/(mol*K)]', 'Universal gas constant');
model.param.set('H', '0.83[mol/m^3/(mol/m^3)]', 'Dimensionless Henry constant');
model.param.set('ci_sweep', '3.5E-4', 'Ambient [CO2]');
model.param.set('ci', 'ci_sweep*P/R/T', 'Intercellular CO2 for A-Ci');
model.param.set('ci_d', '244[umol/mol]*P/R/T');
model.param.set('O', '210[mmol/mol]*P', 'M&M constant of Rubisco for O2');
model.param.set('Dg', '0.135*(T/273.15[K])^1.75*1e-4[m^2/s]', 'Diffusion coefficient of CO2 in pore');
model.param.set('Dl', '1.89e-9[m^2/s]', 'Diffusion coefficient of CO2');
model.param.set('D_HCO3', '1.17e-9[m^2/s]', 'Diffusion coefficient of bicarbonate');
model.param.set('Depis', '5.608607343079838E-8', 'Diffusivity of CO2 in epidermis');
model.param.set('Dw', '0.2*Dl', 'Diffusivity of CO2 in cell wall');
model.param.set('h_coeff', '3.5e-3[m/s]', 'Permeability to CO2 of biological membranes');
model.param.set('h_chloro', 'h_coeff/2', 'CO2 permeability of chl envelope');
model.param.set('h_HCO3', '5.6e-7[m/s]', 'Permeability to bicarbonate');
model.param.set('Vc_max', '274[umol/m^2/s]', 'Maximum capacity of RuBP carboxylation');
model.param.set('jm', '223.2e-6', 'Rate of electron transport');
model.param.set('Tp', '13.6[umol/m^2/s]', 'Trios phosphate utilization');
model.param.set('kmC', '267[umol/mol]*P/R/T*H', 'M&M constant of Rubisco for CO2');
model.param.set('kmO', '164[mmol/mol]*P', 'M&M constant of Rubisco for O2');
model.param.set('ratio_O_Km', 'O/kmO', 'The ratio of [O2] to kmO');
model.param.set('Rd', '2.65[umol/m^2/s]', 'Rate of day-respiration');
model.param.set('gama', '(0.5/3260)*O/R/T*H', 'gama star');
model.param.set('k1', '0.039[1/s]', 'Rate constant of hydration of CO2');
model.param.set('k2', '23[1/s]', 'Rate constant of dehydration of bicarbonate');
model.param.set('ka1', '2.5e-4[mol/l]', 'Equilibrium constant of CO2 (de)hydration');
model.param.set('protonHc', '10^(-7.8)[mol/l]', 'pH, chloroplasr');
model.param.set('tcw', '0.2[um]', 'Cell wall thickness');
model.param.set('tcw_s', '0.2[um]', 'Cell wall thickness of spongy');
model.param.set('tmem', '0.02[um]', 'Plasma membrane thickness');
model.param.set('kcc', '1.5[mol/m^3]','M&M constant of CA');
model.param.set('xa', '0.27[mol/m^3]','Concetrionatio of CA');
model.param.set('ka', '2e5[1/s]','Turn-over rate of CA');
model.param.set('keq', '5.6e-7[mol/l]','Equilib. const. of CA');
model.param.set('kh', '34[mol/m^3]','M&M constant of CA');
model.param.set('tcyt', '0.212[um]','Thickness of cytosol');
model.param.set('fII', '0.50','Thcikness of cytosol');
model.param.set('Phim', '0.85','maximum quantum efficiency at limiting light');
model.param.set('k2LL', '0.357','Conversion efficiency of light to electron transport');
model.param.set('teta', '0.797','convexity factor');
model.param.set('I_inc', '1500[umol/m^2/s]','Thcikness of cytosol');
model.param.set('J', '1.1479451129999999E-4[mol/(m^2*s)]', 'Parametric sweep value');
model.param.set('Beta', '0.85', 'Absorption coeff of leaf');


% Photosynthsis kinetics and diffusion equations
model.variable.create('var1');
model.variable('var1').model('mod1');
model.variable('var1').selection.geom('geom1', 2);
model.variable('var1').set('Ac', 'Vc_max*(c1)/(c1+kmC*(1+ratio_O_Km))');
model.variable('var1').set('Aj', 'J_m*(c1)/(4*c1+8*gama)');
model.variable('var1').set('Ap', '3*Tp/(max(eps,1-(gama/(c1))))');
model.variable('var1').set('AG', 'min(Ac,Aj)');
model.variable('var1').set('A', 'min(AG,Ap)');
model.variable('var1').set('hh', 'ka*xa*(c1-c2*protonHc/keq)/(kcc+kcc*c2/kh+c1)');
model.variable('var1').set('rpc', 'Vc_max*gama/(c1+kmC*(1+ratio_O_Km))');
model.variable('var1').set('rpj', 'J_m*(gama)/(4*c1+8*gama)');
model.variable('var1').set('rp', 'min(rpc,rpj)');
model.variable('var1').set('conc', 'c1');
model.variable('var1').set('concH', 'c2');
model.variable('var1').set('Anet', 'A-Rd-rp');
model.variable('var1').set('ji', 'Phim*fII*I_inc*Beta');
model.variable('var1').set('J_m', '(k2LL*I_inc+jm-sqrt((k2LL*I_inc+jm)^2-4*teta*k2LL*I_inc*jm))/(2*teta)');
model.variable('var1').set('f_chloro', 'Vmeso/Vtissue');

model.view.create('view2', 2);

model.variable.create('var2');
model.variable('var2').model('mod1');
model.variable('var2').selection.geom('geom1', 2);

model.cpl.create('intop1', 'Integration', 'geom1');
model.cpl.create('intop2', 'Integration', 'geom1');

model.physics.create('tds', 'DilutedSpecies', 'geom1');
model.physics('tds').prop('TransportMechanism').set('Convection', '0');
model.physics('tds').field('concentration').field('c1');
model.physics('tds').field('concentration').component({'c1' 'c2'});
model.physics('tds').feature('cdm1').set('D_c2', {'D_HCO3' '0' '0' '0' 'D_HCO3' '0' '0' '0' 'D_HCO3'});
model.physics('tds').feature('cdm1').set('D_c1', {'Dl' '0' '0' '0' 'Dl' '0' '0' '0' 'Dl'});
% model.physics('tds').selection.named('geom1_csel2_dom');

model.physics('tds').create('reac1', 'Reactions', 2);
model.physics('tds').create('reac2', 'Reactions', 2);

M1=model.physics('tds').feature('reac2').selection.named('geom1_csel2_dom').inputEntities(); % palisade
M2=model.physics('tds').feature('reac2').selection.named('geom1_csel3_dom').inputEntities(); % spongy

% Idenitifying the boundaries of each cell, cell types using box selection
N=model.geom('geom1').getNDomains();
if length(M1)==N % 
    core_pali=[];
for i=1:length(CellvertexIds)
    if ismember(i,paliIds)
    x=coordinate(CellvertexIds{i},1);
    y=coordinate(CellvertexIds{i},2); 
    core_pali=[core_pali;x y];
    end
end

xminp=min(core_pali(:,1));
yminp=min(core_pali(:,2))+8;

xmaxp=max(core_pali(:,1));
ymaxp=max(core_pali(:,2))-8;

model.geom('geom1').create('boxsel5', 'BoxSelection');
model.geom('geom1').feature('boxsel5').set('xmax', num2str(xmaxp));
model.geom('geom1').feature('boxsel5').set('xmin', num2str(xminp));
model.geom('geom1').feature('boxsel5').set('ymin', num2str(yminp));
model.geom('geom1').feature('boxsel5').set('ymax', num2str(ymaxp));
model.geom('geom1').selection.create('csel10', 'CumulativeSelection');
model.geom('geom1').selection('csel10').label('pali');
model.geom('geom1').feature('boxsel5').set('contributeto', 'csel10');
model.geom('geom1').run('boxsel5');
M1=model.physics('tds').feature('reac2').selection.named('geom1_csel10_dom').inputEntities(); %
end

Meso=union(M2,M1); % 

 model.physics('tds').selection.set(Meso);
 model.variable('var1').selection.set(Meso);
 model.variable('var2').selection.set(Meso);
 model.cpl('intop1').selection.set(Meso);
 model.variable('var2').set('Vmeso', 'intop1(1)');
 
 yy=min(coordinate(:))-50;
 yy2=min(coordinate(:))-2;
 % lower part of the rectangle
model.geom('geom1').create('boxsel2', 'BoxSelection');
model.geom('geom1').feature('boxsel2').set('xmax', num2str(Xmax_));
model.geom('geom1').feature('boxsel2').set('xmin', num2str(Xmin_));
model.geom('geom1').feature('boxsel2').set('ymin', num2str(yy));
model.geom('geom1').feature('boxsel2').set('ymax', num2str(yy2));
model.geom('geom1').selection.create('csel5', 'CumulativeSelection');
model.geom('geom1').selection('csel5').label('lower rect');
model.geom('geom1').feature('boxsel2').set('contributeto', 'csel5');
model.geom('geom1').run('boxsel2');

Ymax2=max(coordinate(:,2));
Xmin=min(coordinate(:,1));
% Upper part of the rectangle
model.geom('geom1').create('boxsel3', 'BoxSelection');
model.geom('geom1').feature('boxsel3').set('xmax', num2str(Xmax_));
model.geom('geom1').feature('boxsel3').set('xmin', num2str(Xmin));
model.geom('geom1').feature('boxsel3').set('ymin', num2str(Ymax2+2));
model.geom('geom1').feature('boxsel3').set('ymax', num2str(Ymax2+50));
model.geom('geom1').selection.create('csel6', 'CumulativeSelection');
model.geom('geom1').selection('csel6').label('Upper rect');
model.geom('geom1').feature('boxsel3').set('contributeto', 'csel6');
model.geom('geom1').run('boxsel3');

% 
id22=mphselectbox(model,'geom1',[-inf -inf;inf inf]','domain');
M3=model.physics('tds').feature('reac2').selection.named('geom1_csel5_dom').inputEntities(); % 
M4=model.physics('tds').feature('reac2').selection.named('geom1_csel6_dom').inputEntities(); % 
Extra_geoms=union(M3,M4);
Tissue=setdiff(id22,Extra_geoms);

model.variable.create('var3');
model.variable('var3').model('mod1');
model.variable('var3').selection.geom('geom1', 2);
model.variable('var3').selection.set(Tissue);
model.variable('var3').set('Vtissue', 'intop2(1)');
model.cpl('intop2').selection.set(Tissue);

L=Xmax_-Xmin_;
L=L*1e-6; % m
string=['Vtissue/',num2str(L)];

model.variable('var3').set('d', string); % average thickness of the tissue

model.physics('tds').feature('reac1').selection.named('geom1_csel2_dom');
model.physics('tds').feature('reac1').set('R_c2', 'hh');
model.physics('tds').feature('reac1').set('R_c1', '-Anet/(d*f_chloro)-hh');
model.physics('tds').feature('reac2').selection.named('geom1_csel3_dom');
model.physics('tds').feature('reac2').set('R_c2', 'hh');
model.physics('tds').feature('reac2').set('R_c1', '-Anet/(d*f_chloro)-hh');
model.physics('tds').create('fl1', 'Fluxes', 1);
model.physics('tds').feature('fl1').set('FluxType', 'ExternalForcedConvection');
model.physics('tds').feature('fl1').set('species', {'1'; '0'});
model.physics('tds').feature('fl1').set('kc', {'1/(1/h_coeff+tcw/Dw+1/h_chloro+tcyt/Dl)'; '0'});
model.physics('tds').feature('fl1').set('cb', {'ci*H'; '0'});
model.physics('tds').create('tdb1', 'ThinDiffusionBarrier', 1);
model.physics('tds').feature('tdb1').selection.all; % all
model.physics('tds').feature('tdb1').set('ds', '2*(tcw+tmem)');
model.physics('tds').feature('tdb1').set('Ds', {'2*Dw+2*h_coeff*tmem'; '2*h_HCO3*tmem+2*D_HCO3'});
model.physics('tds').feature('init1').setIndex('initc', 'eps', 0);
model.physics('tds').create('tdb2', 'ThinDiffusionBarrier', 1);
model.physics('tds').feature('tdb2').set('ds', '2*(tcw_s+tmem)');
model.physics('tds').feature('tdb2').set('Ds', {'2*Dw+2*h_coeff*tmem'; '2*h_HCO3*tmem+2*D_HCO3'});
model.physics('tds').create('tdb3', 'ThinDiffusionBarrier', 1);
model.physics('tds').feature('tdb3').set('ds', 'tcw+tcw_s+2*tmem');
model.physics('tds').feature('tdb3').set('Ds', {'2*Dw+2*h_coeff*tmem'; '2*h_HCO3*tmem+2*D_HCO3'});

% Finding exposed meso boundary
Xmin1= min(coordinate(:,1))+0.1;
Xmax1= max(coordinate(:,1))-0.2;
model.geom('geom1').create('boxsel1', 'BoxSelection');
model.geom('geom1').feature('boxsel1').set('entitydim', '1');
model.geom('geom1').feature('boxsel1').set('xmin', num2str(Xmin1));
model.geom('geom1').feature('boxsel1').set('xmax', num2str(Xmax1));
model.geom('geom1').selection.create('csel4', 'CumulativeSelection');
model.geom('geom1').selection('csel4').label('testRbnd');
model.geom('geom1').feature('boxsel1').set('contributeto', 'csel4');
model.geom('geom1').run;
model.geom('geom1').run('boxsel1');

h1=model.physics('tds').feature('fl1').selection.named('geom1_csel1_bnd').inputEntities(); % epi+RL
h2=model.physics('tds').feature('fl1').selection.named('geom1_csel2_bnd').inputEntities(); % pali+RL
h3=model.physics('tds').feature('fl1').selection.named('geom1_csel3_bnd').inputEntities(); % spongy+RL
h4=model.physics('tds').feature('fl1').selection.named('geom1_csel4_bnd').inputEntities(); % all except RL

total=union(h1,h2);
total2=union(total,h3);
total3=setdiff(total2,h4); % Right (R) and left (L) boundies of the rectangle

index=setdiff(h2,total3); %  pali except the right boundary of the rectangle

total4=setdiff(index,total3); % exposed pali_epi_upper+Palisade  

index_=setdiff(h3,h2); % exposed spongy  + spongy_epi_lower + R&L boundaries

total5=setdiff(index_,total3); % exposed pali_spongy+spongy+epi_lower_spongy  

index1_=intersect(h2,h1); % Pali_epi_upper interface
index2_=intersect(h3,h1); % Spongy_epi_lower interface

total6=setdiff(total5,index2_); % exposed pali_spongy+ exposed spongy_epi + spongy

expo1=union(total6,total4);
expo=setdiff(expo1,index1_);

model.physics('tds').feature('fl1').selection.set(expo);% exposed 

% mphviewselection(model,'geom1',expo_pali_2,'boundary')
hh=intersect(h1,h2); % pali epi interface
hh2=intersect(h2,h3); % spongy pali interface

expo_pali_int=setdiff(h2,hh); % pali bnd except exposed_pali_epi

expo_spongy=setdiff(h3,hh2); % spongy bnd except the pali_spongy interface

% expo=union(expo_pali_int,expo_spongy);
expo_pali_2=setdiff(expo_pali_int,hh2); % pali internal+ exposed +RL - pali_epi and  spongy_epi interface 

model.physics('tds').feature('tdb1').selection.set(expo_pali_2); % all
model.physics('tds').feature('tdb2').selection.set(expo_spongy); % all
model.physics('tds').feature('tdb3').selection.set(hh2); % all

model.study.create('std2');
% model.study('std2').create('param1', 'Parametric');
model.study('std2').create('stat', 'Stationary');

model.study.create('std1');
model.study('std1').create('stat', 'Stationary');
model.study('std1').feature('stat').activate('tds', true);

model.sol.create('sol1');
model.sol('sol1').study('std1');

model.study('std1').feature('stat').set('notlistsolnum', 1);
model.study('std1').feature('stat').set('notsolnum', '1');
model.study('std1').feature('stat').set('listsolnum', 1);
model.study('std1').feature('stat').set('solnum', '1');

model.sol('sol1').create('st1', 'StudyStep');
model.sol('sol1').feature('st1').set('study', 'std1');
model.sol('sol1').feature('st1').set('studystep', 'stat');
model.sol('sol1').create('v1', 'Variables');
model.sol('sol1').feature('v1').set('control', 'stat');
model.sol('sol1').create('s1', 'Stationary');
model.sol('sol1').feature('s1').create('fc1', 'FullyCoupled');
model.sol('sol1').feature('s1').feature('fc1').set('dtech', 'auto');
model.sol('sol1').feature('s1').feature('fc1').set('maxiter', 50);
model.sol('sol1').feature('s1').feature('fc1').set('initstep', 0.01);
model.sol('sol1').feature('s1').feature('fc1').set('minstep', 1.0E-6);
model.sol('sol1').feature('s1').create('d1', 'Direct');
model.sol('sol1').feature('s1').feature('fc1').set('linsolver', 'd1');
model.sol('sol1').feature('s1').feature('fc1').set('dtech', 'auto');
model.sol('sol1').feature('s1').feature('fc1').set('maxiter', 50);
model.sol('sol1').feature('s1').feature('fc1').set('initstep', 0.01);
model.sol('sol1').feature('s1').feature('fc1').set('minstep', 1.0E-6);
model.sol('sol1').feature('s1').create('p1', 'Parametric');
model.sol('sol1').feature('s1').feature('p1').set('pname', {'ci_sweep'});
model.sol('sol1').feature('s1').feature('p1').set('plistarr', {'54.03649337, 77.2741493, 130.1631251, 184.1316069, 239.4225968, 429.9634441, 608.3342773, 788.646187, 974.8097509, 1338.08416, 1724.306284'});
model.sol('sol1').feature('s1').feature('p1').set('punit', {'umol/mol'});
model.sol('sol1').feature('s1').feature('p1').set('sweeptype', 'sparse');
model.sol('sol1').feature('s1').feature('p1').set('preusesol', 'no');
model.sol('sol1').feature('s1').feature('p1').set('pcontinuationmode', 'no');
model.sol('sol1').feature('s1').feature('p1').set('plot', 'off');
model.sol('sol1').attach('std1');

model.study('std1').create('param', 'Parametric');
model.study('std1').feature('param').setIndex('pname', 'P', 0);
model.study('std1').feature('param').setIndex('plistarr', '', 0);
model.study('std1').feature('param').setIndex('punit', '', 0);
model.study('std1').feature('param').setIndex('pname', 'P', 0);
model.study('std1').feature('param').setIndex('plistarr', '', 0);
model.study('std1').feature('param').setIndex('punit', '', 0);
model.study('std1').feature('param').setIndex('pname', 'ci_sweep', 0);
model.study('std1').feature('param').setIndex('plistarr', '54.03649337, 77.2741493, 130.1631251, 184.1316069, 239.4225968, 429.9634441, 608.3342773, 788.646187, 974.8097509, 1338.08416, 1724.306284', 0);
model.study('std1').feature('param').setIndex('punit', 'umol/mol', 0);
% 

model.mesh.create('mesh1', 'geom1');
model.mesh('mesh1').autoMeshSize(3);
model.mesh('mesh1').run;

model.sol('sol1').runAll;

model.study('std2').create('param', 'Parametric');

model.study('std1').feature('param').setIndex('pname', 'P', 0);
model.study('std1').feature('param').setIndex('plistarr', '', 0);
model.study('std1').feature('param').setIndex('punit', '', 0);
model.study('std1').feature('param').setIndex('pname', 'P', 0);
model.study('std1').feature('param').setIndex('plistarr', '', 0);
model.study('std1').feature('param').setIndex('punit', '', 0);
model.study('std1').feature('param').setIndex('pname', 'ci_sweep', 0);
model.study('std2').feature('param').setIndex('pname', 'P', 0);
model.study('std2').feature('param').setIndex('plistarr', '', 0);
model.study('std2').feature('param').setIndex('punit', '', 0);
model.study('std2').feature('param').setIndex('pname', 'P', 0);
model.study('std2').feature('param').setIndex('plistarr', '', 0);
model.study('std2').feature('param').setIndex('punit', '', 0);
% parametric sweep to solve the model for various Iinc and Ci values
model.study('std1').feature('param').setIndex('plistarr', '54.03649337, 77.2741493, 130.1631251, 184.1316069, 239.4225968, 429.9634441, 608.3342773, 788.646187, 974.8097509, 1338.08416, 1724.306284', 0);
model.study('std1').feature('param').setIndex('punit', 'umol/mol', 0);
model.study('std2').feature('param').setIndex('pname', 'ci_sweep', 0);
model.study('std2').feature('param').setIndex('plistarr', '404.8941252,392.9087017,366.6411861,348.2094222,307.4157621,286.4205483,271.4775008,264.5839562,247.6041048', 0);
model.study('std2').feature('param').setIndex('punit', 'umol/mol', 0);
model.study('std2').feature('param').setIndex('pname', 'P', 1);
model.study('std2').feature('param').setIndex('plistarr', '', 1);
model.study('std2').feature('param').setIndex('punit', '', 1);
model.study('std2').feature('param').setIndex('pname', 'P', 1);
model.study('std2').feature('param').setIndex('plistarr', '', 1);
model.study('std2').feature('param').setIndex('punit', '', 1);
model.study('std2').feature('param').setIndex('pname', 'I_inc', 1);
model.study('std2').feature('param').setIndex('plistarr', '24.43005753,50.69877625,100.0078634,150.1953735,301.3746033,501.2943624,750.8420817,999.7507731,1499.285156', 1);
model.study('std2').feature('param').setIndex('punit', 'umol/m^2/s', 1);

model.sol.create('sol2');
model.sol('sol2').study('std2');

model.study('std2').feature('stat').set('notlistsolnum', 1);
model.study('std2').feature('stat').set('notsolnum', '1');
model.study('std2').feature('stat').set('listsolnum', 1);
model.study('std2').feature('stat').set('solnum', '1');

model.study('std1').label('[CO2] response');
model.study('std2').label('Light response');


model.sol('sol2').create('st1', 'StudyStep');
model.sol('sol2').feature('st1').set('study', 'std2');
model.sol('sol2').feature('st1').set('studystep', 'stat');
model.sol('sol2').create('v1', 'Variables');
model.sol('sol2').feature('v1').set('control', 'stat');
model.sol('sol2').create('s1', 'Stationary');
model.sol('sol2').feature('s1').create('p1', 'Parametric');
model.sol('sol2').feature('s1').feature('p1').set('pname', {'ci_sweep' 'I_inc'});
model.sol('sol2').feature('s1').feature('p1').set('plistarr', {'404.8941252,392.9087017,366.6411861,348.2094222,307.4157621,286.4205483,271.4775008,264.5839562,247.6041048' '24.43005753,50.69877625,100.0078634,150.1953735,301.3746033,501.2943624,750.8420817,999.7507731,1499.285156'});
model.sol('sol2').feature('s1').feature('p1').set('punit', {'umol/mol' 'umol/m^2/s'});
model.sol('sol2').feature('s1').feature('p1').set('sweeptype', 'sparse');
model.sol('sol2').feature('s1').feature('p1').set('preusesol', 'no');
model.sol('sol2').feature('s1').feature('p1').set('pcontinuationmode', 'no');
model.sol('sol2').feature('s1').feature('p1').set('plot', 'off');
model.sol('sol2').feature('s1').feature('p1').set('plotgroup', 'Default');
model.sol('sol2').feature('s1').feature('p1').set('probesel', 'all');
model.sol('sol2').feature('s1').feature('p1').set('probes', {});
model.sol('sol2').feature('s1').set('control', 'stat');
model.sol('sol2').feature('s1').create('fc1', 'FullyCoupled');
model.sol('sol2').feature('s1').feature('fc1').set('dtech', 'auto');
model.sol('sol2').feature('s1').feature('fc1').set('maxiter', 50);
model.sol('sol2').feature('s1').feature('fc1').set('initstep', 0.01);
model.sol('sol2').feature('s1').feature('fc1').set('minstep', 1.0E-6);
model.sol('sol2').feature('s1').create('d1', 'Direct');
model.sol('sol2').feature('s1').feature('fc1').set('linsolver', 'd1');
model.sol('sol2').feature('s1').feature('fc1').set('dtech', 'auto');
model.sol('sol2').feature('s1').feature('fc1').set('maxiter', 50);
model.sol('sol2').feature('s1').feature('fc1').set('initstep', 0.01);
model.sol('sol2').feature('s1').feature('fc1').set('minstep', 1.0E-6);

model.sol('sol2').runAll;

model.study('std1').feature('stat').set('notlistsolnum', 1);
model.study('std1').feature('stat').set('notsolnum', '1');
model.study('std1').feature('stat').set('listsolnum', 1);
model.study('std1').feature('stat').set('solnum', '1');

model.result.create('pg1', 'PlotGroup2D');
model.result('pg1').label('Concentration (tds)');
model.result('pg1').set('data', 'dset1');
model.result('pg1').set('oldanalysistype', 'noneavailable');
model.result('pg1').feature.create('surf1', 'Surface');
model.result('pg1').feature('surf1').label('Surface 1.1');
model.result('pg1').feature('surf1').set('oldanalysistype', 'noneavailable');
model.result('pg1').feature('surf1').set('descractive', true);
model.result('pg1').feature('surf1').set('data', 'parent');

model.result.numerical.create('int1', 'IntSurface');
model.result.numerical('int1').set('expr', 'Anet/Vmeso[m^2]*1e6');
% model.result.numerical('int1').selection.named('geom1_csel2_dom');
model.result.numerical('int1').selection.set(Meso);

model.result.table.create('tbl1', 'Table');
model.result.table('tbl1').comments('Surface Integration 1 (Anet/Vmeso[m^2]*1e6)');
model.result.numerical('int1').set('table', 'tbl1');
model.result.numerical('int1').setResult;

model.result('pg1').run;
model.result('pg1').feature('surf1').set('expr', 'c1*R*T/H/P*1e6');
model.result('pg1').run;
model.result.numerical('int1').set('table', 'tbl1');
model.result.numerical('int1').setResult;
model.result.numerical('int1').set('data', 'dset1');
model.result.numerical('int1').set('table', 'tbl1');
model.result.numerical('int1').setResult;
model.result.numerical.create('int2', 'IntSurface');
model.result.create('pg3', 'PlotGroup2D');
model.result('pg3').label('Concentration (tds) 1');
model.result('pg3').set('data', 'dset2');
model.result('pg3').set('oldanalysistype', 'noneavailable');
model.result('pg3').set('solvertype', 'none');
model.result('pg3').set('solnum', 1);
model.result('pg3').set('showlooplevel', {'off' 'off' 'off'});
model.result('pg3').set('oldanalysistype', 'noneavailable');
model.result('pg3').set('data', 'dset2');
model.result('pg3').feature.create('surf1', 'Surface');
model.result('pg3').feature('surf1').label('Surface 1.1');
model.result('pg3').feature('surf1').set('oldanalysistype', 'noneavailable');
model.result('pg3').feature('surf1').set('solvertype', 'none');
model.result('pg3').feature('surf1').set('descractive', true);
model.result('pg3').feature('surf1').set('data', 'parent');


model.sol('sol2').runAll;

model.result('pg3').run;
model.result('pg1').run;
model.result('pg3').run;
model.result('pg3').feature('surf1').set('expr', 'c1*R*T/H/P*1e6');
model.result('pg3').run;
model.result('pg3').feature('surf1').set('data', 'dset2');
model.result('pg3').run;
model.result.numerical('int2').set('data', 'dset2');
model.result.numerical('int2').selection.set(Meso);

% calculating the response of photosynthesis to Iinc and Ci
model.result.numerical('int2').set('expr', 'Anet/Vmeso[m^2]*1e6');
model.result.table.create('tbl2', 'Table');
model.result.table('tbl2').comments('Surface Integration 2 (Anet/Vmeso[m^2]*1e6)');
model.result.numerical('int2').set('table', 'tbl2');
model.result.numerical('int2').setResult;
model.result.numerical.create('int3', 'IntSurface');
model.result.numerical('int3').set('data', 'dset1');
model.result.numerical('int3').selection.set(Meso);

model.result.numerical('int3').set('expr', 'c1/Vmeso[m^2]*R*T/H/P*1e6');
model.result.table.create('tbl3', 'Table');
model.result.table('tbl3').comments('Surface Integration 3 (c1/Vmeso[m^2]*R*T/H/P)');
model.result.numerical('int3').set('table', 'tbl3');
model.result.numerical('int3').setResult;
model.result.numerical.create('int4', 'IntSurface');
model.result.numerical('int4').set('data', 'dset2');
model.result.numerical('int4').selection.set(Meso);

model.result.numerical('int4').set('expr', 'c1/Vmeso[m^2]*R*T/H/P*1e6');
model.result.table.create('tbl4', 'Table');
model.result.table('tbl4').comments('Surface Integration 4 (c1/Vmeso[m^2]*1e6*R*T/H/P)');
model.result.numerical('int4').set('table', 'tbl4');
model.result.numerical('int4').setResult;

model.result.create('pg8', 'PlotGroup1D');
model.result('pg8').create('tblp1', 'Table');
model.result('pg8').feature('tblp1').set('table', 'tbl1');
model.result('pg8').feature('tblp1').set('xaxisdata', '1');

model.result.numerical('int4').set('data', 'dset2');
model.result.table('tbl4').clearTableData;
model.result.numerical('int4').set('table', 'tbl4');
model.result.numerical('int4').setResult;
model.result('pg8').feature('tblp1').set('plotcolumninput', 'manual');
model.result('pg8').feature('tblp1').set('plotcolumns', {'2'});
model.result('pg8').feature('tblp1').set('xaxisdata', '1');
model.result('pg8').run;

model.result.create('pg10', 'PlotGroup1D');
model.result('pg10').run;
model.result('pg10').set('data', 'dset2');
model.result('pg10').create('tblp1', 'Table');
model.result('pg10').feature('tblp1').set('table', 'tbl2');
model.result('pg10').feature('tblp1').set('xaxisdata', '2');
model.result('pg10').feature('tblp1').set('plotcolumninput', 'manual');
model.result('pg10').feature('tblp1').set('plotcolumns', {'3'});
model.result('pg10').run;

model.result.create('pg11', 'PlotGroup1D');
model.result('pg11').run;
model.result('pg11').create('tblp1', 'Table');
model.result('pg11').feature('tblp1').set('table', 'tbl3');
model.result('pg11').feature('tblp1').set('xaxisdata', '1');
model.result('pg11').feature('tblp1').set('plotcolumninput', 'manual');
model.result('pg11').feature('tblp1').set('plotcolumns', {'2'});
model.result('pg11').run;



model.result.create('pg12', 'PlotGroup1D');
model.result('pg12').run;
model.result('pg12').set('data', 'dset2');
model.result('pg12').create('tblp1', 'Table');
model.result('pg12').feature('tblp1').set('table', 'tbl4');
model.result('pg12').feature('tblp1').set('xaxisdata', '2');
model.result('pg12').feature('tblp1').set('plotcolumninput', 'manual');
model.result('pg12').feature('tblp1').set('plotcolumns', {'3'});
model.result('pg12').run;

model.result('pg8').label('A_Ci');
model.result('pg10').run;
model.result('pg10').label('A_Iinc');
model.result('pg11').run;
model.result('pg11').label('Cc_Ci');
model.result('pg12').run;
model.result('pg12').label('Cc_Iinc');

    % A-Ci
    A_mod=model.result.table('tbl1').getTableData(false);
    A_mod=str2num(A_mod);
    A_mod=reshape(A_mod, 11, []);
    A_Ci_mod=A_mod(:,2);
    
    % A-Iinc
    A_mod2=model.result.table('tbl2').getTableData(false);
    A_mod2=str2num(A_mod2);
    A_mod2=reshape(A_mod2, 9, []);
    A_I_mod=A_mod2(:,3);
    
    % A-Ci
    Cc_mod=model.result.table('tbl3').getTableData(false);
    Cc_mod=str2num(Cc_mod);
    Cc_mod=reshape(Cc_mod, 11, []);
    Cc_Ci_mod=Cc_mod(:,2);

    % A-Iinc
    Cc_mod2=model.result.table('tbl4').getTableData(false);
    Cc_mod2=str2num(Cc_mod2);
    Cc_mod2=reshape(Cc_mod2, 9, []);
    Cc_I_mod=Cc_mod2(:,3);

    result_A_Ci=[result_A_Ci;A_Ci_mod Cc_Ci_mod ];
    result_A_Iinc=[result_A_Iinc;A_I_mod Cc_I_mod];
    
    save result_A_Ci_repair result_A_Ci
    save result_A_Iinc_repair result_A_Iinc
    mphsave(model,model_name);

    ModelUtil.remove(model_name);

out = model;